@@ -1,5 +1,5 @@
 /* Window.java --
-   Copyright (C) 1999, 2000, 2002, 2003, 2004  Free Software Foundation
+   Copyright (C) 1999, 2000, 2002, 2003, 2004, 2005  Free Software Foundation
 
 This file is part of GNU Classpath.
 
@@ -39,8 +39,6 @@ exception statement from your version. */
 package java.awt;
 
 import java.awt.event.ComponentEvent;
-import java.awt.event.FocusEvent;
-import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
 import java.awt.event.WindowFocusListener;
 import java.awt.event.WindowListener;
@@ -75,11 +73,13 @@ public class Window extends Container implements Accessible
   private String warningString = null;
   private int windowSerializedDataVersion = 0; // FIXME
   /** @since 1.2 */
-  // private FocusManager focusMgr;  // FIXME: what is this?  
+  // private FocusManager focusMgr;  // FIXME: what is this?
   /** @since 1.2 */
   private int state = 0;
   /** @since 1.4 */
   private boolean focusableWindowState = true;
+  /** @since 1.5 */
+  private boolean alwaysOnTop = false;
 
   // A list of other top-level windows owned by this window.
   private transient Vector ownedWindows = new Vector();
@@ -87,13 +87,12 @@ public class Window extends Container implements Accessible
   private transient WindowListener windowListener;
   private transient WindowFocusListener windowFocusListener;
   private transient WindowStateListener windowStateListener;
-  private transient GraphicsConfiguration graphicsConfiguration;
 
   private transient boolean shown;
 
   // This is package-private to avoid an accessor method.
   transient Component windowFocusOwner;
-  
+
   /*
    * The number used to generate the name returned by getName.
    */
@@ -107,7 +106,7 @@ public class Window extends Container implements Accessible
     {
       return AccessibleRole.WINDOW;
     }
-    
+
     public AccessibleStateSet getAccessibleStateSet()
     {
       AccessibleStateSet states = super.getAccessibleStateSet();
@@ -117,7 +116,7 @@ public class Window extends Container implements Accessible
     }
   }
 
-  /** 
+  /**
    * This (package access) constructor is used by subclasses that want
    * to build windows that do not have parents.  Eg. toplevel
    * application frames.  Subclasses cannot call super(null), since
@@ -131,41 +130,14 @@ public class Window extends Container implements Accessible
     focusCycleRoot = true;
     setLayout(new BorderLayout());
 
-    addWindowFocusListener (new WindowAdapter ()
-      {
-        public void windowGainedFocus (WindowEvent event)
-        {
-          if (windowFocusOwner != null)
-            {
-              // FIXME: move this section and the other similar
-              // sections in Component into a separate method.
-              EventQueue eq = Toolkit.getDefaultToolkit ().getSystemEventQueue ();
-              synchronized (eq)
-                {
-                  KeyboardFocusManager manager = KeyboardFocusManager.getCurrentKeyboardFocusManager ();
-                  Component currentFocusOwner = manager.getGlobalPermanentFocusOwner ();
-                  if (currentFocusOwner != null)
-                    {
-                      eq.postEvent (new FocusEvent (currentFocusOwner, FocusEvent.FOCUS_LOST,
-                                                    false, windowFocusOwner));
-                      eq.postEvent (new FocusEvent (windowFocusOwner, FocusEvent.FOCUS_GAINED,
-                                                    false, currentFocusOwner));
-                    }
-                  else
-                    eq.postEvent (new FocusEvent (windowFocusOwner, FocusEvent.FOCUS_GAINED, false));
-                }
-            }
-        }
-      });
-    
     GraphicsEnvironment g = GraphicsEnvironment.getLocalGraphicsEnvironment();
-    graphicsConfiguration = g.getDefaultScreenDevice().getDefaultConfiguration();
+    graphicsConfig = g.getDefaultScreenDevice().getDefaultConfiguration();
   }
 
   Window(GraphicsConfiguration gc)
   {
     this();
-    graphicsConfiguration = gc;
+    graphicsConfig = gc;
   }
 
   /**
@@ -185,7 +157,7 @@ public class Window extends Container implements Accessible
 
   /**
    * Initializes a new instance of <code>Window</code> with the specified
-   * parent.  The window will initially be invisible.   
+   * parent.  The window will initially be invisible.
    *
    * @exception IllegalArgumentException If the owner's GraphicsConfiguration
    * is not from a screen device, or if owner is null; this exception is always
@@ -197,10 +169,10 @@ public class Window extends Container implements Accessible
   {
     this (owner, owner.getGraphicsConfiguration ());
   }
-  
+
   /**
    * Initializes a new instance of <code>Window</code> with the specified
-   * parent.  The window will initially be invisible.   
+   * parent.  The window will initially be invisible.
    *
    * @exception IllegalArgumentException If owner is null or if gc is not from a
    * screen device; this exception is always thrown when
@@ -214,10 +186,10 @@ public class Window extends Container implements Accessible
 
     synchronized (getTreeLock())
       {
-	if (owner == null)
-	  throw new IllegalArgumentException ("owner must not be null");
+        if (owner == null)
+          throw new IllegalArgumentException ("owner must not be null");
 
-	parent = owner;
+        parent = owner;
         owner.ownedWindows.add(new WeakReference(this));
       }
 
@@ -231,19 +203,11 @@ public class Window extends Container implements Accessible
       throw new IllegalArgumentException ("gc must be from a screen device");
 
     if (gc == null)
-      graphicsConfiguration = GraphicsEnvironment.getLocalGraphicsEnvironment()
-                                                 .getDefaultScreenDevice()
-                                                 .getDefaultConfiguration();
+      graphicsConfig = GraphicsEnvironment.getLocalGraphicsEnvironment()
+                                          .getDefaultScreenDevice()
+                                          .getDefaultConfiguration();
     else
-      graphicsConfiguration = gc;
-  }
-
-  GraphicsConfiguration getGraphicsConfigurationImpl()
-  {
-    if (graphicsConfiguration != null)
-	return graphicsConfiguration;
-
-    return super.getGraphicsConfigurationImpl();
+      graphicsConfig = gc;
   }
 
   /**
@@ -277,73 +241,89 @@ public class Window extends Container implements Accessible
   /**
    * Shows on-screen this window and any of its owned windows for whom
    * isVisible returns true.
+   * @specnote: Deprecated starting in 1.5.
    */
+  @Deprecated
   public void show()
   {
     synchronized (getTreeLock())
-    {
-    if (parent != null && !parent.isDisplayable())
-      parent.addNotify();
-    if (peer == null)
-      addNotify();
-
-    // Show visible owned windows.
-	Iterator e = ownedWindows.iterator();
-	while(e.hasNext())
-	  {
-	    Window w = (Window)(((Reference) e.next()).get());
-	    if (w != null)
-	      {
-		if (w.isVisible())
-		  w.getPeer().setVisible(true);
-	      }
-     	    else
-	      // Remove null weak reference from ownedWindows.
-	      // Unfortunately this can't be done in the Window's
-	      // finalize method because there is no way to guarantee
-	      // synchronous access to ownedWindows there.
-	      e.remove();
-	  }
-    validate();
-    super.show();
-    toFront();
-
-    KeyboardFocusManager manager = KeyboardFocusManager.getCurrentKeyboardFocusManager ();
-    manager.setGlobalFocusedWindow (this);
-    
-    if (!shown)
       {
-        FocusTraversalPolicy policy = getFocusTraversalPolicy ();
-        Component initialFocusOwner = null;
+        if (peer == null)
+          addNotify();
 
-        if (policy != null)
-          initialFocusOwner = policy.getInitialComponent (this);
+        validate();
+        if (visible)
+          toFront();
+        else
+          {
+            super.show();
+            // Show visible owned windows.
+            Iterator e = ownedWindows.iterator();
+            while (e.hasNext())
+              {
+                Window w = (Window) (((Reference) e.next()).get());
+                if (w != null)
+                  {
+                    if (w.isVisible())
+                      w.getPeer().setVisible(true);
+                  }
+                else
+                  // Remove null weak reference from ownedWindows.
+                  // Unfortunately this can't be done in the Window's
+                  // finalize method because there is no way to guarantee
+                  // synchronous access to ownedWindows there.
+                  e.remove();
+              }
+          }
+        KeyboardFocusManager manager = KeyboardFocusManager.getCurrentKeyboardFocusManager();
+        manager.setGlobalFocusedWindow(this);
 
-        if (initialFocusOwner != null)
-          initialFocusOwner.requestFocusInWindow ();
+        if (! shown)
+          {
+            FocusTraversalPolicy policy = getFocusTraversalPolicy();
+            Component initialFocusOwner = null;
 
-        shown = true;
+            if (policy != null)
+              initialFocusOwner = policy.getInitialComponent(this);
+
+            if (initialFocusOwner != null)
+              initialFocusOwner.requestFocusInWindow();
+
+            // Post WINDOW_OPENED from here.
+            if (windowListener != null
+                || (eventMask & AWTEvent.WINDOW_EVENT_MASK) != 0)
+              {
+                WindowEvent ev = new WindowEvent(this,
+                                                 WindowEvent.WINDOW_OPENED);
+                Toolkit tk = Toolkit.getDefaultToolkit();
+                tk.getSystemEventQueue().postEvent(ev);
+              }
+            shown = true;
+          }
       }
-    }
   }
 
+  /**
+   * @specnote: Deprecated starting in 1.5.
+   */
+  @Deprecated
   public void hide()
   {
     // Hide visible owned windows.
     synchronized (getTreeLock ())
       {
-	Iterator e = ownedWindows.iterator();
-	while(e.hasNext())
-	  {
-	    Window w = (Window)(((Reference) e.next()).get());
-	    if (w != null)
-	      {
-		if (w.isVisible() && w.getPeer() != null)
-		  w.getPeer().setVisible(false);
-	      }
-     	    else
-	      e.remove();
-	  }
+        Iterator e = ownedWindows.iterator();
+        while(e.hasNext())
+          {
+            Window w = (Window)(((Reference) e.next()).get());
+            if (w != null)
+              {
+                if (w.isVisible() && w.getPeer() != null)
+                  w.getPeer().setVisible(false);
+              }
+            else
+              e.remove();
+          }
       }
     super.hide();
   }
@@ -358,37 +338,47 @@ public class Window extends Container implements Accessible
 
     synchronized (getTreeLock ())
       {
-	Iterator e = ownedWindows.iterator();
-	while(e.hasNext())
-	  {
-	    Window w = (Window)(((Reference) e.next()).get());
-	    if (w != null)
-	      w.dispose();
-	    else
-	      // Remove null weak reference from ownedWindows.
-	      e.remove();
-	  }
+        Iterator e = ownedWindows.iterator();
+        while(e.hasNext())
+          {
+            Window w = (Window)(((Reference) e.next()).get());
+            if (w != null)
+              w.dispose();
+            else
+              // Remove null weak reference from ownedWindows.
+              e.remove();
+          }
 
-	for (int i = 0; i < ncomponents; ++i)
-	  component[i].removeNotify();
-	this.removeNotify();
+        for (int i = 0; i < ncomponents; ++i)
+          component[i].removeNotify();
+        this.removeNotify();
 
-        // Post a WINDOW_CLOSED event.
-        WindowEvent we = new WindowEvent(this, WindowEvent.WINDOW_CLOSED);
-        getToolkit().getSystemEventQueue().postEvent(we);
+        // Post WINDOW_CLOSED from here.
+        if (windowListener != null
+            || (eventMask & AWTEvent.WINDOW_EVENT_MASK) != 0)
+          {
+            WindowEvent ev = new WindowEvent(this,
+                                             WindowEvent.WINDOW_CLOSED);
+            Toolkit tk = Toolkit.getDefaultToolkit();
+            tk.getSystemEventQueue().postEvent(ev);
+          }
       }
   }
 
   /**
    * Sends this window to the back so that all other windows display in
    * front of it.
+   *
+   * If the window is set to be always-on-top, this will remove its
+   * always-on-top status.
    */
   public void toBack()
   {
     if (peer != null)
       {
-	WindowPeer wp = (WindowPeer) peer;
-	wp.toBack();
+        if( alwaysOnTop )
+          setAlwaysOnTop( false );
+        ( (WindowPeer) peer ).toBack();
       }
   }
 
@@ -399,10 +389,7 @@ public class Window extends Container implements Accessible
   public void toFront()
   {
     if (peer != null)
-      {
-        WindowPeer wp = (WindowPeer) peer;
-        wp.toFront();
-      }
+      ( (WindowPeer) peer ).toFront();
   }
 
   /**
@@ -410,12 +397,12 @@ public class Window extends Container implements Accessible
    *
    * @return The toolkit used to create this window.
    *
-   * @specnote Unlike Component.getToolkit, this implementation always 
+   * @specnote Unlike Component.getToolkit, this implementation always
    *           returns the value of Toolkit.getDefaultToolkit().
    */
   public Toolkit getToolkit()
   {
-    return Toolkit.getDefaultToolkit();    
+    return Toolkit.getDefaultToolkit();
   }
 
   /**
@@ -468,28 +455,28 @@ public class Window extends Container implements Accessible
     Window [] trimmedList;
     synchronized (getTreeLock ())
       {
-	// Windows with non-null weak references in ownedWindows.
-	Window [] validList = new Window [ownedWindows.size()];
+        // Windows with non-null weak references in ownedWindows.
+        Window [] validList = new Window [ownedWindows.size()];
 
-	Iterator e = ownedWindows.iterator();
-	int numValid = 0;
-	while (e.hasNext())
-	  {
-	    Window w = (Window)(((Reference) e.next()).get());
-	    if (w != null)
-	      validList[numValid++] = w;
-	    else
-	      // Remove null weak reference from ownedWindows.
-	      e.remove();
-	  }
+        Iterator e = ownedWindows.iterator();
+        int numValid = 0;
+        while (e.hasNext())
+          {
+            Window w = (Window)(((Reference) e.next()).get());
+            if (w != null)
+              validList[numValid++] = w;
+            else
+              // Remove null weak reference from ownedWindows.
+              e.remove();
+          }
 
-	if (numValid != validList.length)
-	  {
-	    trimmedList = new Window [numValid];
-	    System.arraycopy (validList, 0, trimmedList, 0, numValid);
-	  }
-	else
-	  trimmedList = validList;
+        if (numValid != validList.length)
+          {
+            trimmedList = new Window [numValid];
+            System.arraycopy (validList, 0, trimmedList, 0, numValid);
+          }
+        else
+          trimmedList = validList;
       }
     return trimmedList;
   }
@@ -502,7 +489,11 @@ public class Window extends Container implements Accessible
    */
   public synchronized void addWindowListener(WindowListener listener)
   {
-    windowListener = AWTEventMulticaster.add(windowListener, listener);
+    if (listener != null)
+      {
+        newEventsOnly = true;
+        windowListener = AWTEventMulticaster.add(windowListener, listener);
+      }
   }
 
   /**
@@ -540,7 +531,7 @@ public class Window extends Container implements Accessible
       AWTEventMulticaster.getListeners(windowFocusListener,
                                        WindowFocusListener.class);
   }
-  
+
   /**
    * Returns an array of all the window state listeners registered on this
    * window.
@@ -559,9 +550,14 @@ public class Window extends Container implements Accessible
    */
   public void addWindowFocusListener (WindowFocusListener wfl)
   {
-    windowFocusListener = AWTEventMulticaster.add (windowFocusListener, wfl);
+    if (wfl != null)
+      {
+        newEventsOnly = true;
+        windowFocusListener = AWTEventMulticaster.add (windowFocusListener,
+                                                       wfl);
+      }
   }
-  
+
   /**
    * Adds the specified listener to this window.
    *
@@ -569,9 +565,14 @@ public class Window extends Container implements Accessible
    */
   public void addWindowStateListener (WindowStateListener wsl)
   {
-    windowStateListener = AWTEventMulticaster.add (windowStateListener, wsl);  
+    if (wsl != null)
+      {
+        newEventsOnly = true;
+        windowStateListener = AWTEventMulticaster.add (windowStateListener,
+                                                       wsl);
+      }
   }
-  
+
   /**
    * Removes the specified listener from this window.
    */
@@ -579,7 +580,7 @@ public class Window extends Container implements Accessible
   {
     windowFocusListener = AWTEventMulticaster.remove (windowFocusListener, wfl);
   }
-  
+
   /**
    * Removes the specified listener from this window.
    *
@@ -600,27 +601,21 @@ public class Window extends Container implements Accessible
    *
    * @since 1.3
    */
-  public EventListener[] getListeners(Class listenerType)
+  public <T extends EventListener> T[] getListeners(Class<T> listenerType)
   {
     if (listenerType == WindowListener.class)
-      return getWindowListeners();
+      return (T[]) getWindowListeners();
     return super.getListeners(listenerType);
   }
 
   void dispatchEventImpl(AWTEvent e)
   {
-    // Make use of event id's in order to avoid multiple instanceof tests.
-    if (e.id <= WindowEvent.WINDOW_LAST 
-        && e.id >= WindowEvent.WINDOW_FIRST
-        && (windowListener != null
-	    || windowFocusListener != null
-	    || windowStateListener != null
-	    || (eventMask & AWTEvent.WINDOW_EVENT_MASK) != 0))
-      processEvent(e);
-    else if (e.id == ComponentEvent.COMPONENT_RESIZED)
-      validate ();
-    else
-      super.dispatchEventImpl(e);
+    if (e.getID() == ComponentEvent.COMPONENT_RESIZED)
+      {
+        invalidate();
+        validate();
+      }
+    super.dispatchEventImpl(e);
   }
 
   /**
@@ -634,7 +629,28 @@ public class Window extends Container implements Accessible
   protected void processEvent(AWTEvent evt)
   {
     if (evt instanceof WindowEvent)
-      processWindowEvent((WindowEvent) evt);
+      {
+        WindowEvent we = (WindowEvent) evt;
+        switch (evt.getID())
+          {
+          case WindowEvent.WINDOW_OPENED:
+          case WindowEvent.WINDOW_CLOSED:
+          case WindowEvent.WINDOW_CLOSING:
+          case WindowEvent.WINDOW_ICONIFIED:
+          case WindowEvent.WINDOW_DEICONIFIED:
+          case WindowEvent.WINDOW_ACTIVATED:
+          case WindowEvent.WINDOW_DEACTIVATED:
+            processWindowEvent(we);
+            break;
+          case WindowEvent.WINDOW_GAINED_FOCUS:
+          case WindowEvent.WINDOW_LOST_FOCUS:
+            processWindowFocusEvent(we);
+            break;
+          case WindowEvent.WINDOW_STATE_CHANGED:
+            processWindowStateEvent(we);
+            break;
+          }
+      }
     else
       super.processEvent(evt);
   }
@@ -649,58 +665,39 @@ public class Window extends Container implements Accessible
    */
   protected void processWindowEvent(WindowEvent evt)
   {
-    int id = evt.getID();
-
-    if (id == WindowEvent.WINDOW_GAINED_FOCUS
-	|| id == WindowEvent.WINDOW_LOST_FOCUS)
-      processWindowFocusEvent (evt);
-    else if (id == WindowEvent.WINDOW_STATE_CHANGED)
-      processWindowStateEvent (evt);
-    else
+    if (windowListener != null)
       {
-	if (windowListener != null)
-	  {
-	    switch (evt.getID())
-	      {
-	      case WindowEvent.WINDOW_ACTIVATED:
-		windowListener.windowActivated(evt);
-		break;
-
-	      case WindowEvent.WINDOW_CLOSED:
-		windowListener.windowClosed(evt);
-		break;
-
-	      case WindowEvent.WINDOW_CLOSING:
-		windowListener.windowClosing(evt);
-		break;
-
-	      case WindowEvent.WINDOW_DEACTIVATED:
-		windowListener.windowDeactivated(evt);
-		break;
-
-	      case WindowEvent.WINDOW_DEICONIFIED:
-		windowListener.windowDeiconified(evt);
-		break;
-
-	      case WindowEvent.WINDOW_ICONIFIED:
-		windowListener.windowIconified(evt);
-		break;
-
-	      case WindowEvent.WINDOW_OPENED:
-		windowListener.windowOpened(evt);
-		break;
-
-	      default:
-		break;
-	      }
-	  }
+        switch (evt.getID())
+          {
+          case WindowEvent.WINDOW_ACTIVATED:
+            windowListener.windowActivated(evt);
+            break;
+          case WindowEvent.WINDOW_CLOSED:
+            windowListener.windowClosed(evt);
+            break;
+          case WindowEvent.WINDOW_CLOSING:
+            windowListener.windowClosing(evt);
+            break;
+          case WindowEvent.WINDOW_DEACTIVATED:
+            windowListener.windowDeactivated(evt);
+            break;
+          case WindowEvent.WINDOW_DEICONIFIED:
+            windowListener.windowDeiconified(evt);
+            break;
+          case WindowEvent.WINDOW_ICONIFIED:
+            windowListener.windowIconified(evt);
+            break;
+          case WindowEvent.WINDOW_OPENED:
+            windowListener.windowOpened(evt);
+            break;
+          }
       }
   }
-  
+
   /**
    * Identifies if this window is active.  The active window is a Frame or
    * Dialog that has focus or owns the active window.
-   *  
+   *
    * @return true if active, else false.
    * @since 1.4
    */
@@ -713,7 +710,7 @@ public class Window extends Container implements Accessible
   /**
    * Identifies if this window is focused.  A window is focused if it is the
    * focus owner or it contains the focus owner.
-   * 
+   *
    * @return true if focused, else false.
    * @since 1.4
    */
@@ -722,7 +719,7 @@ public class Window extends Container implements Accessible
     KeyboardFocusManager manager = KeyboardFocusManager.getCurrentKeyboardFocusManager ();
     return manager.getFocusedWindow() == this;
   }
-  
+
   /**
    * Returns the child window that has focus if this window is active.
    * This method returns <code>null</code> if this window is not active
@@ -807,7 +804,7 @@ public class Window extends Container implements Accessible
   {
     int x = 0;
     int y = 0;
-    
+
     if (c == null || !c.isShowing())
       {
         GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
@@ -823,7 +820,7 @@ public class Window extends Container implements Accessible
 
         x = c.getLocationOnScreen().x;
         y = c.getLocationOnScreen().y;
-        
+
         // If bottom of component is cut off, window placed
         // on the left or the right side of component
         if ((y + cHeight) > screenSize.height)
@@ -856,7 +853,7 @@ public class Window extends Container implements Accessible
               x = 0;
             else
               x += (cWidth - width) / 2;
-            
+
             y += (cHeight - height) / 2;
           }
         else
@@ -895,9 +892,9 @@ public class Window extends Container implements Accessible
     WindowBltBufferStrategy(int numBuffers, boolean accelerated)
     {
       super(numBuffers,
-	    new BufferCapabilities(new ImageCapabilities(accelerated),
-				   new ImageCapabilities(accelerated),
-				   BufferCapabilities.FlipContents.COPIED));
+            new BufferCapabilities(new ImageCapabilities(accelerated),
+                                   new ImageCapabilities(accelerated),
+                                   BufferCapabilities.FlipContents.COPIED));
     }
   }
 
@@ -918,9 +915,9 @@ public class Window extends Container implements Accessible
       throws AWTException
     {
       super(numBuffers,
-	    new BufferCapabilities(new ImageCapabilities(true),
-				   new ImageCapabilities(true),
-				   BufferCapabilities.FlipContents.COPIED));
+            new BufferCapabilities(new ImageCapabilities(true),
+                                   new ImageCapabilities(true),
+                                   BufferCapabilities.FlipContents.COPIED));
     }
   }
 
@@ -951,18 +948,18 @@ public class Window extends Container implements Accessible
   {
     if (numBuffers < 1)
       throw new IllegalArgumentException("Window.createBufferStrategy: number"
-					 + " of buffers is less than one");
+                                         + " of buffers is less than one");
 
     if (!isDisplayable())
       throw new IllegalStateException("Window.createBufferStrategy: window is"
-				      + " not displayable");
+                                      + " not displayable");
 
     BufferStrategy newStrategy = null;
 
     // try a flipping strategy
     try
       {
-	newStrategy = new WindowFlipBufferStrategy(numBuffers);
+        newStrategy = new WindowFlipBufferStrategy(numBuffers);
       }
     catch (AWTException e)
       {
@@ -999,11 +996,11 @@ public class Window extends Container implements Accessible
   {
     if (numBuffers < 1)
       throw new IllegalArgumentException("Window.createBufferStrategy: number"
-					 + " of buffers is less than one");
+                                         + " of buffers is less than one");
 
     if (caps == null)
       throw new IllegalArgumentException("Window.createBufferStrategy:"
-					 + " capabilities object is null");
+                                         + " capabilities object is null");
 
     // a flipping strategy was requested
     if (caps.isPageFlipping())
@@ -1026,11 +1023,11 @@ public class Window extends Container implements Accessible
   /**
    * @since 1.2
    *
-   * @deprecated
+   * @deprecated replaced by Component.applyComponentOrientation.
    */
   public void applyResourceBundle(ResourceBundle rb)
   {
-    throw new Error ("Not implemented");
+    applyComponentOrientation(ComponentOrientation.getOrientation(rb));
   }
 
   /**
@@ -1043,7 +1040,7 @@ public class Window extends Container implements Accessible
     ResourceBundle rb = ResourceBundle.getBundle(rbName, Locale.getDefault(),
       ClassLoader.getSystemClassLoader());
     if (rb != null)
-      applyResourceBundle(rb);    
+      applyResourceBundle(rb);
   }
 
   /**
@@ -1060,16 +1057,21 @@ public class Window extends Container implements Accessible
     return accessibleContext;
   }
 
-  /** 
+  /**
    * Get graphics configuration.  The implementation for Window will
    * not ask any parent containers, since Window is a toplevel
    * window and not actually embedded in the parent component.
    */
   public GraphicsConfiguration getGraphicsConfiguration()
   {
-    if (graphicsConfiguration != null) return graphicsConfiguration;
-    if (peer != null) return peer.getGraphicsConfiguration();
-    return null;
+    GraphicsConfiguration conf = graphicsConfig;
+    if (conf == null)
+      {
+        conf = GraphicsEnvironment.getLocalGraphicsEnvironment()
+        .getDefaultScreenDevice().getDefaultConfiguration();
+        graphicsConfig = conf;
+      }
+    return conf;
   }
 
   protected void processWindowFocusEvent(WindowEvent event)
@@ -1081,17 +1083,17 @@ public class Window extends Container implements Accessible
           case WindowEvent.WINDOW_GAINED_FOCUS:
             windowFocusListener.windowGainedFocus (event);
             break;
-            
+
           case WindowEvent.WINDOW_LOST_FOCUS:
             windowFocusListener.windowLostFocus (event);
             break;
-            
+
           default:
             break;
           }
       }
   }
-  
+
   /**
    * @since 1.4
    */
@@ -1120,10 +1122,10 @@ public class Window extends Container implements Accessible
 
     return false;
   }
-  
+
   /**
    * Returns the value of the focusableWindowState property.
-   * 
+   *
    * @since 1.4
    */
   public boolean getFocusableWindowState ()
@@ -1133,7 +1135,7 @@ public class Window extends Container implements Accessible
 
   /**
    * Sets the value of the focusableWindowState property.
-   * 
+   *
    * @since 1.4
    */
   public void setFocusableWindowState (boolean focusableWindowState)
@@ -1141,6 +1143,96 @@ public class Window extends Container implements Accessible
     this.focusableWindowState = focusableWindowState;
   }
 
+  /**
+   * Check whether this Container is a focus cycle root.
+   * Returns always <code>true</code> as Windows are the
+   * root of the focus cycle.
+   *
+   * @return Always <code>true</code>.
+   *
+   * @since 1.4
+   */
+  public final boolean isFocusCycleRoot()
+  {
+    return true;
+  }
+
+  /**
+   * Set whether or not this Container is the root of a focus
+   * traversal cycle. Windows are the root of the focus cycle
+   * and therefore this method does nothing.
+   *
+   * @param focusCycleRoot ignored.
+   *
+   * @since 1.4
+   */
+  public final void setFocusCycleRoot(boolean focusCycleRoot)
+  {
+    // calls to the method are ignored
+  }
+
+  /**
+   * Returns the root container that owns the focus cycle where this
+   * component resides. Windows have no ancestors and this method
+   * returns always <code>null</code>.
+   *
+   * @return Always <code>null</code>.
+   * @since 1.4
+   */
+  public final Container getFocusCycleRootAncestor()
+  {
+    return null;
+  }
+
+  /**
+   * Returns whether the Windows is an always-on-top window,
+   * meaning whether the window can be obscured by other windows or not.
+   *
+   * @return <code>true</code> if the windows is always-on-top,
+   * <code>false</code> otherwise.
+   * @since 1.5
+   */
+  public final boolean isAlwaysOnTop()
+  {
+    return alwaysOnTop;
+  }
+
+  /**
+   * Sets the always-on-top state of this window (if supported).
+   *
+   * Setting a window to always-on-top means it will not be obscured
+   * by any other windows (with the exception of other always-on-top
+   * windows). Not all platforms may support this.
+   *
+   * If an window's always-on-top status is changed to false, the window
+   * will remain at the front but not be anchored there.
+   *
+   * Calling toBack() on an always-on-top window will change its
+   * always-on-top status to false.
+   *
+   * @since 1.5
+   */
+  public final void setAlwaysOnTop(boolean alwaysOnTop)
+  {
+    SecurityManager sm = System.getSecurityManager();
+    if (sm != null)
+      sm.checkPermission( new AWTPermission("setWindowAlwaysOnTop") );
+
+    if( this.alwaysOnTop == alwaysOnTop )
+      return;
+
+    if( alwaysOnTop )
+      toFront();
+
+    firePropertyChange("alwaysOnTop", this.alwaysOnTop, alwaysOnTop );
+    this.alwaysOnTop = alwaysOnTop;
+
+    if (peer != null)
+      ( (WindowPeer) peer).updateAlwaysOnTop();
+    else
+      System.out.println("Null peer?!");
+  }
+
   /**
    * Generate a unique name for this window.
    *
@@ -1151,6 +1243,42 @@ public class Window extends Container implements Accessible
     return "win" + getUniqueLong();
   }
 
+  /**
+   * Overridden to handle WindowEvents.
+   *
+   * @return <code>true</code> when the specified event type is enabled,
+   *         <code>false</code> otherwise
+   */
+  boolean eventTypeEnabled(int type)
+  {
+    boolean enabled = false;
+    switch (type)
+      {
+        case WindowEvent.WINDOW_OPENED:
+        case WindowEvent.WINDOW_CLOSED:
+        case WindowEvent.WINDOW_CLOSING:
+        case WindowEvent.WINDOW_ICONIFIED:
+        case WindowEvent.WINDOW_DEICONIFIED:
+        case WindowEvent.WINDOW_ACTIVATED:
+        case WindowEvent.WINDOW_DEACTIVATED:
+          enabled = ((eventMask & AWTEvent.WINDOW_EVENT_MASK) != 0)
+                    || windowListener != null;
+          break;
+        case WindowEvent.WINDOW_GAINED_FOCUS:
+        case WindowEvent.WINDOW_LOST_FOCUS:
+          enabled = ((eventMask & AWTEvent.WINDOW_FOCUS_EVENT_MASK) != 0)
+                    || windowFocusListener != null;
+          break;
+        case WindowEvent.WINDOW_STATE_CHANGED:
+          enabled = ((eventMask & AWTEvent.WINDOW_STATE_EVENT_MASK) != 0)
+                    || windowStateListener != null;
+          break;
+        default:
+          enabled = super.eventTypeEnabled(type);
+      }
+    return enabled;
+  }
+
   private static synchronized long getUniqueLong()
   {
     return next_window_number++;
